@@ -14,13 +14,11 @@ if [[ ! -f configure.self ]]; then
     exit 1
 fi
 
-_DOCKER_IMAGES="centos:6 centos:7 centos:8"
+_DOCKER_IMAGES="rockylinux:8 rockylinux:9"
 _RPMDIR=artifacts
 
 if [[ -n $1 ]]; then
     _RPMDIR="$1"
-    echo "Usage: $0 <path/to/rpmdirectory>"
-    exit 1
 fi
 
 _RPMDIR=$(readlink -f $_RPMDIR)
