@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2018 Magnus Edenhill
+ * Copyright (c) 2018-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,17 +47,17 @@
  *
  */
 
-static void rd_kafka_idemp_pid_timer_restart (rd_kafka_t *rk,
-                                              rd_bool_t immediate,
-                                              const char *reason);
+static void rd_kafka_idemp_pid_timer_restart(rd_kafka_t *rk,
+                                             rd_bool_t immediate,
+                                             const char *reason);
 
 
 /**
  * @brief Set the producer's idempotence state.
  * @locks rd_kafka_wrlock() MUST be held
  */
-void rd_kafka_idemp_set_state (rd_kafka_t *rk,
-                               rd_kafka_idemp_state_t new_state) {
+void rd_kafka_idemp_set_state(rd_kafka_t *rk,
+                              rd_kafka_idemp_state_t new_state) {
 
         if (rk->rk_eos.idemp_state == new_state)
                 return;
@@ -70,8 +70,7 @@ void rd_kafka_idemp_set_state (rd_kafka_t *rk,
                 rd_kafka_dbg(rk, EOS, "IDEMPSTATE",
                              "Denying state change %s -> %s since a "
                              "fatal error has been raised",
-                             rd_kafka_idemp_state2str(rk->rk_eos.
-                                                      idemp_state),
+                             rd_kafka_idemp_state2str(rk->rk_eos.idemp_state),
                              rd_kafka_idemp_state2str(new_state));
                 rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_FATAL_ERROR);
                 return;
@@ -79,11 +78,10 @@ void rd_kafka_idemp_set_state (rd_kafka_t *rk,
 
         rd_kafka_dbg(rk, EOS, "IDEMPSTATE",
                      "Idempotent producer state change %s -> %s",
-                     rd_kafka_idemp_state2str(rk->rk_eos.
-                                              idemp_state),
+                     rd_kafka_idemp_state2str(rk->rk_eos.idemp_state),
                      rd_kafka_idemp_state2str(new_state));
 
-        rk->rk_eos.idemp_state = new_state;
+        rk->rk_eos.idemp_state    = new_state;
         rk->rk_eos.ts_idemp_state = rd_clock();
 
         /* Inform transaction manager of state change */
@@ -93,8 +91,6 @@ void rd_kafka_idemp_set_state (rd_kafka_t *rk,
 
 
 
-
-
 /**
  * @brief Find a usable broker suitable for acquiring Pid
  *        or Coordinator query.
@@ -103,10 +99,10 @@ void rd_kafka_idemp_set_state (rd_kafka_t *rk,
  *
  * @returns a broker with increased refcount, or NULL on error.
  */
-rd_kafka_broker_t *
-rd_kafka_idemp_broker_any (rd_kafka_t *rk,
-                           rd_kafka_resp_err_t *errp,
-                           char *errstr, size_t errstr_size) {
+rd_kafka_broker_t *rd_kafka_idemp_broker_any(rd_kafka_t *rk,
+                                             rd_kafka_resp_err_t *errp,
+                                             char *errstr,
+                                             size_t errstr_size) {
         rd_kafka_broker_t *rkb;
         int up_cnt;
 
@@ -122,15 +118,17 @@ rd_kafka_idemp_broker_any (rd_kafka_t *rk,
                             "%s not supported by "
                             "any of the %d connected broker(s): requires "
                             "Apache Kafka broker version >= 0.11.0",
-                            rd_kafka_is_transactional(rk) ?
-                            "Transactions" : "Idempotent producer",
+                            rd_kafka_is_transactional(rk)
+                                ? "Transactions"
+                                : "Idempotent producer",
                             up_cnt);
         } else {
                 *errp = RD_KAFKA_RESP_ERR__TRANSPORT;
                 rd_snprintf(errstr, errstr_size,
                             "No brokers available for %s (%d broker(s) known)",
-                            rd_kafka_is_transactional(rk) ?
-                            "Transactions" : "Idempotent producer",
+                            rd_kafka_is_transactional(rk)
+                                ? "Transactions"
+                                : "Idempotent producer",
                             rd_atomic32_get(&rk->rk_broker_cnt));
         }
 
@@ -152,14 +150,13 @@ rd_kafka_idemp_broker_any (rd_kafka_t *rk,
  * @locks rd_kafka_wrlock() MUST be held
  * @locality rdkafka main thread
  */
-rd_bool_t rd_kafka_idemp_check_error (rd_kafka_t *rk,
-                                      rd_kafka_resp_err_t err,
-                                      const char *errstr,
-                                      rd_bool_t is_fatal) {
+rd_bool_t rd_kafka_idemp_check_error(rd_kafka_t *rk,
+                                     rd_kafka_resp_err_t err,
+                                     const char *errstr,
+                                     rd_bool_t is_fatal) {
         const char *preface = "";
 
-        switch (err)
-        {
+        switch (err) {
         case RD_KAFKA_RESP_ERR__UNSUPPORTED_FEATURE:
         case RD_KAFKA_RESP_ERR_INVALID_TRANSACTION_TIMEOUT:
         case RD_KAFKA_RESP_ERR_TRANSACTIONAL_ID_AUTHORIZATION_FAILED:
@@ -168,9 +165,10 @@ rd_bool_t rd_kafka_idemp_check_error (rd_kafka_t *rk,
                 break;
 
         case RD_KAFKA_RESP_ERR_INVALID_PRODUCER_EPOCH:
+        case RD_KAFKA_RESP_ERR_PRODUCER_FENCED:
                 is_fatal = rd_true;
                 /* Normalize error */
-                err = RD_KAFKA_RESP_ERR__FENCED;
+                err     = RD_KAFKA_RESP_ERR__FENCED;
                 preface = "Producer fenced by newer instance: ";
                 break;
 
@@ -182,11 +180,11 @@ rd_bool_t rd_kafka_idemp_check_error (rd_kafka_t *rk,
                 return rd_false;
 
         if (rd_kafka_is_transactional(rk))
-                rd_kafka_txn_set_fatal_error(rk, RD_DONT_LOCK,
-                                             err, "%s%s", preface, errstr);
+                rd_kafka_txn_set_fatal_error(rk, RD_DONT_LOCK, err, "%s%s",
+                                             preface, errstr);
         else
-                rd_kafka_set_fatal_error0(rk, RD_DONT_LOCK,
-                                          err, "%s%s", preface, errstr);
+                rd_kafka_set_fatal_error0(rk, RD_DONT_LOCK, err, "%s%s",
+                                          preface, errstr);
 
         rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_FATAL_ERROR);
 
@@ -202,7 +200,7 @@ rd_bool_t rd_kafka_idemp_check_error (rd_kafka_t *rk,
  * @locality rdkafka main thread
  * @locks rd_kafka_wrlock() MUST be held.
  */
-void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
+void rd_kafka_idemp_pid_fsm(rd_kafka_t *rk) {
         rd_kafka_resp_err_t err;
         char errstr[512];
         rd_kafka_broker_t *rkb;
@@ -213,9 +211,8 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
         if (unlikely(rd_kafka_fatal_error_code(rk)))
                 return;
 
- redo:
-        switch (rk->rk_eos.idemp_state)
-        {
+redo:
+        switch (rk->rk_eos.idemp_state) {
         case RD_KAFKA_IDEMP_STATE_INIT:
         case RD_KAFKA_IDEMP_STATE_TERM:
         case RD_KAFKA_IDEMP_STATE_FATAL_ERROR:
@@ -230,7 +227,7 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
                 if (!rd_kafka_is_transactional(rk) ||
                     rk->rk_eos.txn_curr_coord) {
                         rd_kafka_idemp_set_state(
-                                rk, RD_KAFKA_IDEMP_STATE_WAIT_TRANSPORT);
+                            rk, RD_KAFKA_IDEMP_STATE_WAIT_TRANSPORT);
                         goto redo;
                 }
 
@@ -246,20 +243,31 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
         case RD_KAFKA_IDEMP_STATE_WAIT_TRANSPORT:
                 /* Waiting for broker/coordinator to become available */
                 if (rd_kafka_is_transactional(rk)) {
-                        /* Assert that a coordinator has been assigned by
-                         * inspecting txn_curr_coord (the real broker)
-                         * rather than txn_coord (the logical broker). */
-                        rd_assert(rk->rk_eos.txn_curr_coord);
+                        /* Check that a proper coordinator broker has
+                         * been assigned by inspecting txn_curr_coord
+                         * (the real broker) rather than txn_coord
+                         * (the logical broker). */
+                        if (!rk->rk_eos.txn_curr_coord) {
+                                /*
+                                 * Can happen if the coordinator wasn't set or
+                                 * wasn't up initially and has been set to NULL
+                                 * after a COORDINATOR_NOT_AVAILABLE error in
+                                 * FindCoordinatorResponse. When the coordinator
+                                 * is known this FSM will be called again.
+                                 */
+                                rd_kafka_txn_coord_query(
+                                    rk, "Awaiting coordinator");
+                                return;
+                        }
                         rkb = rk->rk_eos.txn_coord;
                         rd_kafka_broker_keep(rkb);
 
                 } else {
-                        rkb = rd_kafka_idemp_broker_any(rk, &err,
-                                                        errstr, sizeof(errstr));
+                        rkb = rd_kafka_idemp_broker_any(rk, &err, errstr,
+                                                        sizeof(errstr));
 
-                        if (!rkb &&
-                            rd_kafka_idemp_check_error(rk, err, errstr,
-                                                       rd_false))
+                        if (!rkb && rd_kafka_idemp_check_error(rk, err, errstr,
+                                                               rd_false))
                                 return; /* Fatal error */
                 }
 
@@ -267,10 +275,9 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
                         /* The coordinator broker monitor will re-trigger
                          * the fsm sooner if txn_coord has a state change,
                          * else rely on the timer to retry. */
-                        rd_kafka_idemp_pid_timer_restart(rk, rd_false,
-                                                         rkb ?
-                                                         "No broker available" :
-                                                         "Coordinator not up");
+                        rd_kafka_idemp_pid_timer_restart(
+                            rk, rd_false,
+                            rkb ? "No broker available" : "Coordinator not up");
 
                         if (rkb)
                                 rd_kafka_broker_destroy(rkb);
@@ -299,14 +306,13 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
                         }
 
                         err = rd_kafka_InitProducerIdRequest(
-                                rkb,
-                                rk->rk_conf.eos.transactional_id,
-                                rk->rk_conf.eos.transaction_timeout_ms,
-                                rd_kafka_pid_valid(rk->rk_eos.pid) ?
-                                &rk->rk_eos.pid : NULL,
-                                errstr+err_of, sizeof(errstr)-err_of,
-                                RD_KAFKA_REPLYQ(rk->rk_ops, 0),
-                                rd_kafka_handle_InitProducerId, NULL);
+                            rkb, rk->rk_conf.eos.transactional_id,
+                            rk->rk_conf.eos.transaction_timeout_ms,
+                            rd_kafka_pid_valid(rk->rk_eos.pid) ? &rk->rk_eos.pid
+                                                               : NULL,
+                            errstr + err_of, sizeof(errstr) - err_of,
+                            RD_KAFKA_REPLYQ(rk->rk_ops, 0),
+                            rd_kafka_handle_InitProducerId, NULL);
 
                         if (err == RD_KAFKA_RESP_ERR__UNSUPPORTED_FEATURE &&
                             rd_kafka_pid_valid(rk->rk_eos.pid))
@@ -315,22 +321,21 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
                         rd_rkb_dbg(rkb, EOS, "GETPID", "Acquiring ProducerId");
 
                         err = rd_kafka_InitProducerIdRequest(
-                                rkb,
-                                NULL,
-                                -1,
-                                NULL,
-                                errstr, sizeof(errstr),
-                                RD_KAFKA_REPLYQ(rk->rk_ops, 0),
-                                rd_kafka_handle_InitProducerId, NULL);
+                            rkb, NULL, -1, NULL, errstr, sizeof(errstr),
+                            RD_KAFKA_REPLYQ(rk->rk_ops, 0),
+                            rd_kafka_handle_InitProducerId, NULL);
+                }
+
+                if (err) {
+                        rd_rkb_dbg(rkb, EOS, "GETPID",
+                                   "Can't acquire ProducerId from "
+                                   "this broker: %s",
+                                   errstr);
                 }
 
                 rd_kafka_broker_destroy(rkb);
 
                 if (err) {
-                        rd_rkb_dbg(rkb, EOS, "GETPID",
-                                   "Can't acquire ProducerId from "
-                                   "this broker: %s", errstr);
-
                         if (rd_kafka_idemp_check_error(rk, err, errstr,
                                                        is_fatal))
                                 return; /* Fatal error */
@@ -362,6 +367,11 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
                 /* Wait for outstanding ProduceRequests to finish
                  * before bumping the current epoch. */
                 break;
+
+        case RD_KAFKA_IDEMP_STATE_WAIT_TXN_ABORT:
+                /* Wait for txnmgr to abort its current transaction
+                 * and then trigger a drain & reset or bump. */
+                break;
         }
 }
 
@@ -372,7 +382,7 @@ void rd_kafka_idemp_pid_fsm (rd_kafka_t *rk) {
  * @locality rdkafka main thread
  * @locks none
  */
-static void rd_kafka_idemp_pid_timer_cb (rd_kafka_timers_t *rkts, void *arg) {
+static void rd_kafka_idemp_pid_timer_cb(rd_kafka_timers_t *rkts, void *arg) {
         rd_kafka_t *rk = arg;
 
         rd_kafka_wrlock(rk);
@@ -389,14 +399,14 @@ static void rd_kafka_idemp_pid_timer_cb (rd_kafka_timers_t *rkts, void *arg) {
  * @locality any
  * @locks none
  */
-static void rd_kafka_idemp_pid_timer_restart (rd_kafka_t *rk,
-                                              rd_bool_t immediate,
-                                              const char *reason) {
+static void rd_kafka_idemp_pid_timer_restart(rd_kafka_t *rk,
+                                             rd_bool_t immediate,
+                                             const char *reason) {
         rd_kafka_dbg(rk, EOS, "TXN", "Starting PID FSM timer%s: %s",
                      immediate ? " (fire immediately)" : "", reason);
-        rd_kafka_timer_start_oneshot(&rk->rk_timers,
-                                     &rk->rk_eos.pid_tmr, rd_true,
-                                     1000 * (immediate ? 1 : 500/*500ms*/),
+        rd_kafka_timer_start_oneshot(&rk->rk_timers, &rk->rk_eos.pid_tmr,
+                                     rd_true,
+                                     1000 * (immediate ? 1 : 500 /*500ms*/),
                                      rd_kafka_idemp_pid_timer_cb, rk);
 }
 
@@ -407,13 +417,13 @@ static void rd_kafka_idemp_pid_timer_restart (rd_kafka_t *rk,
  * @locality rdkafka main thread
  * @locks none
  */
-void rd_kafka_idemp_request_pid_failed (rd_kafka_broker_t *rkb,
-                                        rd_kafka_resp_err_t err) {
+void rd_kafka_idemp_request_pid_failed(rd_kafka_broker_t *rkb,
+                                       rd_kafka_resp_err_t err) {
         rd_kafka_t *rk = rkb->rkb_rk;
         char errstr[512];
 
-        rd_rkb_dbg(rkb, EOS, "GETPID",
-                   "Failed to acquire PID: %s", rd_kafka_err2str(err));
+        rd_rkb_dbg(rkb, EOS, "GETPID", "Failed to acquire PID: %s",
+                   rd_kafka_err2str(err));
 
         if (err == RD_KAFKA_RESP_ERR__DESTROY)
                 return; /* Ignore */
@@ -422,8 +432,8 @@ void rd_kafka_idemp_request_pid_failed (rd_kafka_broker_t *rkb,
 
         rd_snprintf(errstr, sizeof(errstr),
                     "Failed to acquire %s PID from broker %s: %s",
-                    rd_kafka_is_transactional(rk) ?
-                    "transactional" : "idempotence",
+                    rd_kafka_is_transactional(rk) ? "transactional"
+                                                  : "idempotence",
                     rd_kafka_broker_name(rkb), rd_kafka_err2str(err));
 
         rd_kafka_wrlock(rk);
@@ -440,6 +450,8 @@ void rd_kafka_idemp_request_pid_failed (rd_kafka_broker_t *rkb,
              err == RD_KAFKA_RESP_ERR_COORDINATOR_NOT_AVAILABLE))
                 rd_kafka_txn_coord_set(rk, NULL, "%s", errstr);
 
+        /* This error code is read by init_transactions() for propagation
+         * to the application. */
         rk->rk_eos.txn_init_err = err;
 
         rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_REQ_PID);
@@ -459,8 +471,8 @@ void rd_kafka_idemp_request_pid_failed (rd_kafka_broker_t *rkb,
  * @locality rdkafka main thread
  * @locks none
  */
-void rd_kafka_idemp_pid_update (rd_kafka_broker_t *rkb,
-                                const rd_kafka_pid_t pid) {
+void rd_kafka_idemp_pid_update(rd_kafka_broker_t *rkb,
+                               const rd_kafka_pid_t pid) {
         rd_kafka_t *rk = rkb->rkb_rk;
 
         rd_kafka_wrlock(rk);
@@ -477,7 +489,7 @@ void rd_kafka_idemp_pid_update (rd_kafka_broker_t *rkb,
         if (!rd_kafka_pid_valid(pid)) {
                 rd_kafka_wrunlock(rk);
                 rd_rkb_log(rkb, LOG_WARNING, "GETPID",
-                           "Acquired invalid PID{%"PRId64",%hd}: ignoring",
+                           "Acquired invalid PID{%" PRId64 ",%hd}: ignoring",
                            pid.id, pid.epoch);
                 rd_kafka_idemp_request_pid_failed(rkb,
                                                   RD_KAFKA_RESP_ERR__BAD_MSG);
@@ -485,13 +497,12 @@ void rd_kafka_idemp_pid_update (rd_kafka_broker_t *rkb,
         }
 
         if (rd_kafka_pid_valid(rk->rk_eos.pid))
-                rd_kafka_dbg(rk, EOS, "GETPID",
-                             "Acquired %s (previous %s)",
+                rd_kafka_dbg(rk, EOS, "GETPID", "Acquired %s (previous %s)",
                              rd_kafka_pid2str(pid),
                              rd_kafka_pid2str(rk->rk_eos.pid));
         else
-                rd_kafka_dbg(rk, EOS, "GETPID",
-                             "Acquired %s", rd_kafka_pid2str(pid));
+                rd_kafka_dbg(rk, EOS, "GETPID", "Acquired %s",
+                             rd_kafka_pid2str(pid));
         rk->rk_eos.pid = pid;
         rk->rk_eos.epoch_cnt++;
 
@@ -503,7 +514,8 @@ void rd_kafka_idemp_pid_update (rd_kafka_broker_t *rkb,
 
         /* Wake up all broker threads (that may have messages to send
          * that were waiting for a Producer ID). */
-        rd_kafka_all_brokers_wakeup(rk, RD_KAFKA_BROKER_STATE_INIT);
+        rd_kafka_all_brokers_wakeup(rk, RD_KAFKA_BROKER_STATE_INIT,
+                                    "PID updated");
 }
 
 
@@ -514,8 +526,8 @@ void rd_kafka_idemp_pid_update (rd_kafka_broker_t *rkb,
  * @locality any
  * @locks none
  */
-static void rd_kafka_idemp_drain_done (rd_kafka_t *rk) {
-        rd_bool_t restart_tmr = rd_false;
+static void rd_kafka_idemp_drain_done(rd_kafka_t *rk) {
+        rd_bool_t restart_tmr    = rd_false;
         rd_bool_t wakeup_brokers = rd_false;
 
         rd_kafka_wrlock(rk);
@@ -559,8 +571,8 @@ static void rd_kafka_idemp_drain_done (rd_kafka_t *rk) {
         /* Wake up all broker threads (that may have messages to send
          * that were waiting for a Producer ID). */
         if (wakeup_brokers)
-                rd_kafka_all_brokers_wakeup(rk, RD_KAFKA_BROKER_STATE_INIT);
-
+                rd_kafka_all_brokers_wakeup(rk, RD_KAFKA_BROKER_STATE_INIT,
+                                            "message drain done");
 }
 
 /**
@@ -570,7 +582,7 @@ static void rd_kafka_idemp_drain_done (rd_kafka_t *rk) {
  * @locality any
  * @locks none
  */
-static RD_INLINE void rd_kafka_idemp_check_drain_done (rd_kafka_t *rk) {
+static RD_INLINE void rd_kafka_idemp_check_drain_done(rd_kafka_t *rk) {
         if (rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt) == 0)
                 rd_kafka_idemp_drain_done(rk);
 }
@@ -585,14 +597,13 @@ static RD_INLINE void rd_kafka_idemp_check_drain_done (rd_kafka_t *rk) {
  * @locality any
  * @locks none
  */
-void rd_kafka_idemp_drain_reset (rd_kafka_t *rk, const char *reason) {
+void rd_kafka_idemp_drain_reset(rd_kafka_t *rk, const char *reason) {
         rd_kafka_wrlock(rk);
         rd_kafka_dbg(rk, EOS, "DRAIN",
                      "Beginning partition drain for %s reset "
                      "for %d partition(s) with in-flight requests: %s",
                      rd_kafka_pid2str(rk->rk_eos.pid),
-                     rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt),
-                     reason);
+                     rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt), reason);
         rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_DRAIN_RESET);
         rd_kafka_wrunlock(rk);
 
@@ -605,33 +616,74 @@ void rd_kafka_idemp_drain_reset (rd_kafka_t *rk, const char *reason) {
  * @brief Schedule an epoch bump when the local ProduceRequest queues
  *        have been fully drained.
  *
- * The PID is not bumped until the queues are fully drained.
+ * The PID is not bumped until the queues are fully drained and the current
+ * transaction is aborted (if any).
  *
+ * @param allow_txn_abort If this is a transactional producer and this flag is
+ *                        true then we trigger an abortable txn error to abort
+ *                        the current transaction first. The txnmgr will later
+ *                        call us back with this flag set to false to go ahead
+ *                        with the epoch bump.
  * @param fmt is a human-readable reason for the bump
  *
  *
  * @locality any
  * @locks none
  */
-void rd_kafka_idemp_drain_epoch_bump (rd_kafka_t *rk, const char *fmt, ...) {
+void rd_kafka_idemp_drain_epoch_bump0(rd_kafka_t *rk,
+                                      rd_bool_t allow_txn_abort,
+                                      rd_kafka_resp_err_t err,
+                                      const char *fmt,
+                                      ...) {
         va_list ap;
         char buf[256];
+        rd_bool_t requires_txn_abort =
+            allow_txn_abort && rd_kafka_is_transactional(rk);
 
         va_start(ap, fmt);
         rd_vsnprintf(buf, sizeof(buf), fmt, ap);
         va_end(ap);
 
         rd_kafka_wrlock(rk);
-        rd_kafka_dbg(rk, EOS, "DRAIN",
-                     "Beginning partition drain for %s epoch bump "
-                     "for %d partition(s) with in-flight requests: %s",
-                     rd_kafka_pid2str(rk->rk_eos.pid),
-                     rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt), buf);
-        rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_DRAIN_BUMP);
+
+
+        if (requires_txn_abort) {
+                rd_kafka_dbg(rk, EOS, "DRAIN",
+                             "Need transaction abort before beginning "
+                             "partition drain in state %s for %s epoch bump "
+                             "for %d partition(s) with in-flight requests: %s",
+                             rd_kafka_idemp_state2str(rk->rk_eos.idemp_state),
+                             rd_kafka_pid2str(rk->rk_eos.pid),
+                             rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt),
+                             buf);
+                rd_kafka_idemp_set_state(rk,
+                                         RD_KAFKA_IDEMP_STATE_WAIT_TXN_ABORT);
+
+        } else {
+                rd_kafka_dbg(rk, EOS, "DRAIN",
+                             "Beginning partition drain in state %s "
+                             "for %s epoch bump "
+                             "for %d partition(s) with in-flight requests: %s",
+                             rd_kafka_idemp_state2str(rk->rk_eos.idemp_state),
+                             rd_kafka_pid2str(rk->rk_eos.pid),
+                             rd_atomic32_get(&rk->rk_eos.inflight_toppar_cnt),
+                             buf);
+
+                rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_DRAIN_BUMP);
+        }
+
         rd_kafka_wrunlock(rk);
 
-        /* Check right away if the drain could be done. */
-        rd_kafka_idemp_check_drain_done(rk);
+        if (requires_txn_abort) {
+                /* Transactions: bumping the epoch requires the current
+                 *               transaction to be aborted first. */
+                rd_kafka_txn_set_abortable_error_with_bump(rk, err, "%s", buf);
+
+        } else {
+                /* Idempotent producer: check right away if the drain could
+                 *                      be done. */
+                rd_kafka_idemp_check_drain_done(rk);
+        }
 }
 
 /**
@@ -640,13 +692,12 @@ void rd_kafka_idemp_drain_epoch_bump (rd_kafka_t *rk, const char *fmt, ...) {
  * @locks toppar_lock MUST be held
  * @locality broker thread (leader or not)
  */
-void rd_kafka_idemp_drain_toppar (rd_kafka_toppar_t *rktp,
-                                  const char *reason) {
+void rd_kafka_idemp_drain_toppar(rd_kafka_toppar_t *rktp, const char *reason) {
         if (rktp->rktp_eos.wait_drain)
                 return;
 
-        rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, EOS|RD_KAFKA_DBG_TOPIC, "DRAIN",
-                     "%.*s [%"PRId32"] beginning partition drain: %s",
+        rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, EOS | RD_KAFKA_DBG_TOPIC, "DRAIN",
+                     "%.*s [%" PRId32 "] beginning partition drain: %s",
                      RD_KAFKAP_STR_PR(rktp->rktp_rkt->rkt_topic),
                      rktp->rktp_partition, reason);
         rktp->rktp_eos.wait_drain = rd_true;
@@ -659,8 +710,8 @@ void rd_kafka_idemp_drain_toppar (rd_kafka_toppar_t *rktp,
  * @locality any
  * @locks none
  */
-void rd_kafka_idemp_inflight_toppar_sub (rd_kafka_t *rk,
-                                         rd_kafka_toppar_t *rktp) {
+void rd_kafka_idemp_inflight_toppar_sub(rd_kafka_t *rk,
+                                        rd_kafka_toppar_t *rktp) {
         int r = rd_atomic32_sub(&rk->rk_eos.inflight_toppar_cnt, 1);
 
         if (r == 0) {
@@ -680,8 +731,8 @@ void rd_kafka_idemp_inflight_toppar_sub (rd_kafka_t *rk,
  * @locality toppar handler thread
  * @locks none
  */
-void rd_kafka_idemp_inflight_toppar_add (rd_kafka_t *rk,
-                                         rd_kafka_toppar_t *rktp) {
+void rd_kafka_idemp_inflight_toppar_add(rd_kafka_t *rk,
+                                        rd_kafka_toppar_t *rktp) {
         rd_atomic32_add(&rk->rk_eos.inflight_toppar_cnt, 1);
 }
 
@@ -693,13 +744,16 @@ void rd_kafka_idemp_inflight_toppar_add (rd_kafka_t *rk,
  * @locality rdkafka main thread
  * @locks none
  */
-void rd_kafka_idemp_start (rd_kafka_t *rk, rd_bool_t immediate) {
+void rd_kafka_idemp_start(rd_kafka_t *rk, rd_bool_t immediate) {
 
         if (rd_kafka_terminating(rk))
                 return;
 
         rd_kafka_wrlock(rk);
-        rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_REQ_PID);
+        /* Don't restart PID acquisition if there's already an outstanding
+         * request. */
+        if (rk->rk_eos.idemp_state != RD_KAFKA_IDEMP_STATE_WAIT_PID)
+                rd_kafka_idemp_set_state(rk, RD_KAFKA_IDEMP_STATE_REQ_PID);
         rd_kafka_wrunlock(rk);
 
         /* Schedule request timer */
@@ -715,7 +769,7 @@ void rd_kafka_idemp_start (rd_kafka_t *rk, rd_bool_t immediate) {
  * @locality rdkafka main thread
  * @locks none / not needed from rd_kafka_new()
  */
-void rd_kafka_idemp_init (rd_kafka_t *rk) {
+void rd_kafka_idemp_init(rd_kafka_t *rk) {
         rd_assert(thrd_is_current(rk->rk_thread));
 
         rd_atomic32_init(&rk->rk_eos.inflight_toppar_cnt, 0);
@@ -731,7 +785,7 @@ void rd_kafka_idemp_init (rd_kafka_t *rk) {
                  * so just set the state to indicate that we want to
                  * acquire a PID as soon as possible and start
                  * the timer. */
-                rd_kafka_idemp_start(rk, rd_false/*non-immediate*/);
+                rd_kafka_idemp_start(rk, rd_false /*non-immediate*/);
 }
 
 
@@ -741,7 +795,7 @@ void rd_kafka_idemp_init (rd_kafka_t *rk) {
  * @locality rdkafka main thread
  * @locks rd_kafka_wrlock() MUST be held
  */
-void rd_kafka_idemp_term (rd_kafka_t *rk) {
+void rd_kafka_idemp_term(rd_kafka_t *rk) {
         rd_assert(thrd_is_current(rk->rk_thread));
 
         rd_kafka_wrlock(rk);
