@@ -383,6 +383,7 @@ rd_kafka_produceva(rd_kafka_t *rk, const rd_kafka_vu_t *vus, size_t cnt) {
         rd_kafka_error_t *error      = NULL;
         rd_kafka_headers_t *hdrs     = NULL;
         rd_kafka_headers_t *app_hdrs = NULL; /* App-provided headers list */
+        int existing                 = 0;
         size_t i;
 
         if (unlikely(rd_kafka_check_produce(rk, &error)))
@@ -392,8 +393,11 @@ rd_kafka_produceva(rd_kafka_t *rk, const rd_kafka_vu_t *vus, size_t cnt) {
                 const rd_kafka_vu_t *vu = &vus[i];
                 switch (vu->vtype) {
                 case RD_KAFKA_VTYPE_TOPIC:
-                        rkt =
-                            rd_kafka_topic_new0(rk, vu->u.cstr, NULL, NULL, 1);
+                        rkt = rd_kafka_topic_new0(rk, vu->u.cstr, NULL,
+                                                  &existing, 1);
+                        if (!existing)
+                                rd_kafka_topic_fast_leader_query(
+                                    rk, rd_true /* force */);
                         break;
 
                 case RD_KAFKA_VTYPE_RKT:
@@ -549,6 +553,7 @@ rd_kafka_resp_err_t rd_kafka_producev(rd_kafka_t *rk, ...) {
         rd_kafka_resp_err_t err;
         rd_kafka_headers_t *hdrs     = NULL;
         rd_kafka_headers_t *app_hdrs = NULL; /* App-provided headers list */
+        int existing                 = 0;
 
         if (unlikely((err = rd_kafka_check_produce(rk, NULL))))
                 return err;
@@ -559,7 +564,10 @@ rd_kafka_resp_err_t rd_kafka_producev(rd_kafka_t *rk, ...) {
                 switch (vtype) {
                 case RD_KAFKA_VTYPE_TOPIC:
                         rkt = rd_kafka_topic_new0(rk, va_arg(ap, const char *),
-                                                  NULL, NULL, 1);
+                                                  NULL, &existing, 1);
+                        if (!existing)
+                                rd_kafka_topic_fast_leader_query(
+                                    rk, rd_true /* force */);
                         break;
 
                 case RD_KAFKA_VTYPE_RKT:
@@ -2528,22 +2536,28 @@ int unittest_msg(void) {
                                            {10, 10},
                                            {33692865, 33692865},
                                            {0, 0}});
-        fails += unittest_msgq_insert_sort(
-            "many messages", insert_baseline, NULL,
-            (const struct ut_msg_range[]) {{100000, 200000},
-                                           {400000, 450000},
-                                           {900000, 920000},
-                                           {33692864, 33751992},
-                                           {33906868, 33993690},
-                                           {40000000, 44000000},
-                                           {0, 0}},
-            (const struct ut_msg_range[]) {{1, 199},
-                                           {350000, 360000},
-                                           {500000, 500010},
-                                           {1000000, 1000200},
-                                           {33751993, 33906867},
-                                           {50000001, 50000001},
-                                           {0, 0}});
+        if (rd_unittest_with_valgrind) {
+                RD_UT_WARN(
+                    "Skipping large message range test "
+                    "when using Valgrind");
+        } else {
+                fails += unittest_msgq_insert_sort(
+                    "many messages", insert_baseline, NULL,
+                    (const struct ut_msg_range[]) {{100000, 200000},
+                                                   {400000, 450000},
+                                                   {900000, 920000},
+                                                   {33692864, 33751992},
+                                                   {33906868, 33993690},
+                                                   {40000000, 44000000},
+                                                   {0, 0}},
+                    (const struct ut_msg_range[]) {{1, 199},
+                                                   {350000, 360000},
+                                                   {500000, 500010},
+                                                   {1000000, 1000200},
+                                                   {33751993, 33906867},
+                                                   {50000001, 50000001},
+                                                   {0, 0}});
+        }
         fails += unittest_msgq_insert_sort(
             "issue #2508", insert_baseline, NULL,
             (const struct ut_msg_range[]) {
