@@ -1,7 +1,7 @@
 /*
  * librdkafka - The Apache Kafka C/C++ library
  *
- * Copyright (c) 2015 Magnus Edenhill
+ * Copyright (c) 2015-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,39 +32,41 @@
 struct rd_kafka_sasl_provider {
         const char *name;
 
-        /**< Per client-instance (rk) initializer */
-        int  (*init) (rd_kafka_t *rk, char *errstr, size_t errstr_size);
+        /** Per client-instance (rk) initializer */
+        int (*init)(rd_kafka_t *rk, char *errstr, size_t errstr_size);
 
-        /**< Per client-instance (rk) destructor */
-        void (*term) (rd_kafka_t *rk);
+        /** Per client-instance (rk) destructor */
+        void (*term)(rd_kafka_t *rk);
 
-        /**< Returns rd_true if provider is ready to be used, else rd_false */
-        rd_bool_t (*ready) (rd_kafka_t *rk);
+        /** Returns rd_true if provider is ready to be used, else rd_false */
+        rd_bool_t (*ready)(rd_kafka_t *rk);
 
-        int (*client_new) (rd_kafka_transport_t *rktrans,
-                           const char *hostname,
-                           char *errstr, size_t errstr_size);
+        int (*client_new)(rd_kafka_transport_t *rktrans,
+                          const char *hostname,
+                          char *errstr,
+                          size_t errstr_size);
 
-        int (*recv) (struct rd_kafka_transport_s *s,
-                     const void *buf, size_t size,
-                     char *errstr, size_t errstr_size);
-        void (*close) (struct rd_kafka_transport_s *);
+        int (*recv)(struct rd_kafka_transport_s *s,
+                    const void *buf,
+                    size_t size,
+                    char *errstr,
+                    size_t errstr_size);
+        void (*close)(struct rd_kafka_transport_s *);
 
-        void (*broker_init) (rd_kafka_broker_t *rkb);
-        void (*broker_term) (rd_kafka_broker_t *rkb);
+        void (*broker_init)(rd_kafka_broker_t *rkb);
+        void (*broker_term)(rd_kafka_broker_t *rkb);
 
-        int (*conf_validate) (rd_kafka_t *rk,
-                              char *errstr, size_t errstr_size);
+        int (*conf_validate)(rd_kafka_t *rk, char *errstr, size_t errstr_size);
 };
 
-#ifdef _MSC_VER
+#ifdef _WIN32
 extern const struct rd_kafka_sasl_provider rd_kafka_sasl_win32_provider;
 #endif
 
 #if WITH_SASL_CYRUS
 extern const struct rd_kafka_sasl_provider rd_kafka_sasl_cyrus_provider;
-void rd_kafka_sasl_cyrus_global_term (void);
-int rd_kafka_sasl_cyrus_global_init (void);
+void rd_kafka_sasl_cyrus_global_term(void);
+int rd_kafka_sasl_cyrus_global_init(void);
 #endif
 
 extern const struct rd_kafka_sasl_provider rd_kafka_sasl_plain_provider;
@@ -77,9 +79,11 @@ extern const struct rd_kafka_sasl_provider rd_kafka_sasl_scram_provider;
 extern const struct rd_kafka_sasl_provider rd_kafka_sasl_oauthbearer_provider;
 #endif
 
-void rd_kafka_sasl_auth_done (rd_kafka_transport_t *rktrans);
-int rd_kafka_sasl_send (rd_kafka_transport_t *rktrans,
-                        const void *payload, int len,
-                        char *errstr, size_t errstr_size);
+void rd_kafka_sasl_auth_done(rd_kafka_transport_t *rktrans);
+int rd_kafka_sasl_send(rd_kafka_transport_t *rktrans,
+                       const void *payload,
+                       int len,
+                       char *errstr,
+                       size_t errstr_size);
 
 #endif /* _RDKAFKA_SASL_INT_H_ */
