@@ -302,8 +302,12 @@ static void do_test_compaction(int msgs_per_key, const char *compression) {
          * this doesn't really work because the low watermark offset
          * is not updated on compaction if the first segment is not deleted.
          * But it serves as a pause to let compaction kick in
-         * which is triggered by the dummy produce above. */
-        wait_compaction(rk, topic, partition, 0, 20 * 1000);
+         * which is triggered by the dummy produce above.
+         * Compaction timer is every 15 seconds and
+         * with a large number of segments it can
+         * take the same time. */
+        wait_compaction(rk, topic, partition, 0,
+                        msgcnt > 50 ? 30 * 1000 : 20 * 1000);
 
         TEST_SAY(_C_YEL "Verify messages after compaction\n");
         /* After compaction we expect the following messages:
