@@ -1,7 +1,8 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2016, Magnus Edenhill
+ * Copyright (c) 2016-2022, Magnus Edenhill
+ *               2025, Confluent Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,7 +72,7 @@ static void do_test_fetch_max_bytes(void) {
    * but due to batching overhead it would result in situations where
    * the consumer asked for 1000000 bytes and got 1000096 bytes batch, which
    * was higher than the 1000000 limit.
-   * See https://github.com/edenhill/librdkafka/issues/1616
+   * See https://github.com/confluentinc/librdkafka/issues/1616
    *
    * With the added configuration strictness checks, a user-supplied
    * value is no longer over-written:
@@ -87,6 +88,9 @@ static void do_test_fetch_max_bytes(void) {
     Test::Fail("Failed to create KafkaConsumer: " + errstr);
   delete conf;
 
+  /* For next consumer */
+  test_wait_topic_exists(c->c_ptr(), topic.c_str(), 5000);
+
   /* Subscribe */
   std::vector<std::string> topics;
   topics.push_back(topic);
