@@ -1,7 +1,7 @@
 /*
  * sockem - socket-level network emulation
  *
- * Copyright (c) 2016, Magnus Edenhill, Andreas Smas
+ * Copyright (c) 2016-2022, Magnus Edenhill, Andreas Smas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,72 +50,71 @@
 #define socket_errno() WSAGetLastError()
 #else
 #define socket_errno() errno
-#define SOCKET_ERROR -1
+#define SOCKET_ERROR   -1
 #endif
 
 #ifndef strdupa
-#define strdupa(s)                                                      \
-        ({                                                              \
-                const char *_s = (s);                                   \
-                size_t _len = strlen(_s)+1;                             \
-                char *_d = (char *)alloca(_len);                        \
-                (char *)memcpy(_d, _s, _len);                           \
+#define strdupa(s)                                                             \
+        ({                                                                     \
+                const char *_s = (s);                                          \
+                size_t _len    = strlen(_s) + 1;                               \
+                char *_d       = (char *)alloca(_len);                         \
+                (char *)memcpy(_d, _s, _len);                                  \
         })
 #endif
 
 #include <pthread.h>
 typedef pthread_mutex_t mtx_t;
-#define mtx_init(M) pthread_mutex_init(M, NULL)
+#define mtx_init(M)    pthread_mutex_init(M, NULL)
 #define mtx_destroy(M) pthread_mutex_destroy(M)
-#define mtx_lock(M) pthread_mutex_lock(M)
-#define mtx_unlock(M) pthread_mutex_unlock(M)
+#define mtx_lock(M)    pthread_mutex_lock(M)
+#define mtx_unlock(M)  pthread_mutex_unlock(M)
 
 typedef pthread_t thrd_t;
-#define thrd_create(THRD,START_ROUTINE,ARG) \
-  pthread_create(THRD, NULL, START_ROUTINE, ARG)
-#define thrd_join0(THRD)                  \
-        pthread_join(THRD, NULL)
+#define thrd_create(THRD, START_ROUTINE, ARG)                                  \
+        pthread_create(THRD, NULL, START_ROUTINE, ARG)
+#define thrd_join0(THRD) pthread_join(THRD, NULL)
 
 
 static mtx_t sockem_lock;
 static LIST_HEAD(, sockem_s) sockems;
 
 static pthread_once_t sockem_once = PTHREAD_ONCE_INIT;
-static char *sockem_conf_str = "";
+static char *sockem_conf_str      = "";
 
 typedef int64_t sockem_ts_t;
 
 
 #ifdef LIBSOCKEM_PRELOAD
-static int (*sockem_orig_connect) (int, const struct sockaddr *, socklen_t);
-static int (*sockem_orig_close) (int);
+static int (*sockem_orig_connect)(int, const struct sockaddr *, socklen_t);
+static int (*sockem_orig_close)(int);
 
-#define sockem_close0(S)        (sockem_orig_close(S))
-#define sockem_connect0(S,A,AL) (sockem_orig_connect(S,A,AL))
+#define sockem_close0(S)          (sockem_orig_close(S))
+#define sockem_connect0(S, A, AL) (sockem_orig_connect(S, A, AL))
 #else
-#define sockem_close0(S)        close(S)
-#define sockem_connect0(S,A,AL) connect(S,A,AL)
+#define sockem_close0(S)          close(S)
+#define sockem_connect0(S, A, AL) connect(S, A, AL)
 #endif
 
 
 struct sockem_conf {
         /* FIXME: these needs to be implemented */
-        int tx_thruput;  /* app->peer bytes/second */
-        int rx_thruput;  /* peer->app bytes/second */
-        int delay;       /* latency in ms */
-        int jitter;      /* latency variation in ms */
-        int debug;       /* enable sockem printf debugging */
-        size_t recv_bufsz;    /* recv chunk/buffer size */
-        int direct;      /* direct forward, no delay or rate-limiting */
+        int tx_thruput;    /* app->peer bytes/second */
+        int rx_thruput;    /* peer->app bytes/second */
+        int delay;         /* latency in ms */
+        int jitter;        /* latency variation in ms */
+        int debug;         /* enable sockem printf debugging */
+        size_t recv_bufsz; /* recv chunk/buffer size */
+        int direct;        /* direct forward, no delay or rate-limiting */
 };
 
 
 typedef struct sockem_buf_s {
         TAILQ_ENTRY(sockem_buf_s) sb_link;
-        size_t  sb_size;
-        size_t  sb_of;
-        char   *sb_data;
-        int64_t sb_at;  /* Transmit at this absolute time. */
+        size_t sb_size;
+        size_t sb_of;
+        char *sb_data;
+        int64_t sb_at; /* Transmit at this absolute time. */
 } sockem_buf_t;
 
 
@@ -130,46 +129,47 @@ struct sockem_s {
                 SOCKEM_TERM
         } run;
 
-        int as;        /* application's socket. */
-        int ls;        /* internal application listen socket */
-        int ps;        /* internal peer socket connecting sockem to the peer.*/
+        int as; /* application's socket. */
+        int ls; /* internal application listen socket */
+        int ps; /* internal peer socket connecting sockem to the peer.*/
 
-        void *recv_buf;     /* Receive buffer */
-        size_t recv_bufsz;  /* .. size */
+        void *recv_buf;    /* Receive buffer */
+        size_t recv_bufsz; /* .. size */
 
-        int linked;    /* On sockems list */
+        int linked; /* On sockems list */
 
-        thrd_t thrd;   /* Forwarder thread */
+        thrd_t thrd; /* Forwarder thread */
 
-        mtx_t  lock;
+        mtx_t lock;
 
-        struct sockem_conf conf;  /* application-set config.
-                                   * protected by .lock */
+        struct sockem_conf conf; /* application-set config.
+                                  * protected by .lock */
 
-        struct sockem_conf use;   /* last copy of .conf
-                                   * local to skm thread */
+        struct sockem_conf use; /* last copy of .conf
+                                 * local to skm thread */
 
-        TAILQ_HEAD(, sockem_buf_s) bufs; /* Buffers in queue waiting for
-                                          * transmission (delayed) */
+        TAILQ_HEAD(, sockem_buf_s)
+        bufs; /* Buffers in queue waiting for
+               * transmission (delayed) */
 
-        size_t bufs_size; /* Total number of bytes currently enqueued
-                           * for transmission */
+        size_t bufs_size;     /* Total number of bytes currently enqueued
+                               * for transmission */
         size_t bufs_size_max; /* Soft max threshold for bufs_size,
                                * when this value is exceeded the app fd
                                * is removed from the poll set until
                                * bufs_size falls below the threshold again. */
         int poll_fd_cnt;
-        int64_t ts_last_fwd;  /* For rate-limiter: timestamp of last forward */
+        int64_t ts_last_fwd; /* For rate-limiter: timestamp of last forward */
 };
 
 
-static int sockem_vset (sockem_t *skm, va_list ap);
+static int sockem_vset(sockem_t *skm, va_list ap);
 
 
 /**
  * A microsecond monotonic clock
  */
-static __attribute__((unused)) __inline int64_t sockem_clock (void) {
+static __attribute__((unused)) __inline int64_t sockem_clock(void) {
 #ifdef __APPLE__
         /* No monotonic clock on Darwin */
         struct timeval tv;
@@ -181,14 +181,14 @@ static __attribute__((unused)) __inline int64_t sockem_clock (void) {
         struct timespec ts;
         clock_gettime(CLOCK_MONOTONIC, &ts);
         return ((int64_t)ts.tv_sec * 1000000LLU) +
-                ((int64_t)ts.tv_nsec / 1000LLU);
+               ((int64_t)ts.tv_nsec / 1000LLU);
 #endif
 }
 
 /**
  * @brief Initialize libsockem once.
  */
-static void sockem_init (void) {
+static void sockem_init(void) {
         mtx_init(&sockem_lock);
         sockem_conf_str = getenv("SOCKEM_CONF");
         if (!sockem_conf_str)
@@ -198,7 +198,7 @@ static void sockem_init (void) {
                         sockem_conf_str);
 #ifdef LIBSOCKEM_PRELOAD
         sockem_orig_connect = dlsym(RTLD_NEXT, "connect");
-        sockem_orig_close = dlsym(RTLD_NEXT, "close");
+        sockem_orig_close   = dlsym(RTLD_NEXT, "close");
 #endif
 }
 
@@ -207,7 +207,7 @@ static void sockem_init (void) {
  * @returns the maximum waittime in ms for poll(), at most 1000 ms.
  * @remark lock must be held
  */
-static int sockem_calc_waittime (sockem_t *skm, int64_t now) {
+static int sockem_calc_waittime(sockem_t *skm, int64_t now) {
         const sockem_buf_t *sb;
         int64_t r;
 
@@ -229,7 +229,7 @@ static int sockem_calc_waittime (sockem_t *skm, int64_t now) {
 /**
  * @brief Unlink and destroy a buffer
  */
-static void sockem_buf_destroy (sockem_t *skm, sockem_buf_t *sb) {
+static void sockem_buf_destroy(sockem_t *skm, sockem_buf_t *sb) {
         skm->bufs_size -= sb->sb_size - sb->sb_of;
         TAILQ_REMOVE(&skm->bufs, sb, sb_link);
         free(sb);
@@ -238,8 +238,8 @@ static void sockem_buf_destroy (sockem_t *skm, sockem_buf_t *sb) {
 /**
  * @brief Add delayed buffer to transmit.
  */
-static sockem_buf_t *sockem_buf_add (sockem_t *skm,
-                                     size_t size, const void *data) {
+static sockem_buf_t *
+sockem_buf_add(sockem_t *skm, size_t size, const void *data) {
         sockem_buf_t *sb;
 
         skm->bufs_size += size;
@@ -253,10 +253,9 @@ static sockem_buf_t *sockem_buf_add (sockem_t *skm,
 
         sb->sb_of   = 0;
         sb->sb_size = size;
-        sb->sb_data = (char *)(sb+1);
+        sb->sb_data = (char *)(sb + 1);
         sb->sb_at   = sockem_clock() +
-                ((skm->use.delay +
-                  (skm->use.jitter / 2)/*FIXME*/) * 1000);
+                    ((skm->use.delay + (skm->use.jitter / 2) /*FIXME*/) * 1000);
         memcpy(sb->sb_data, data, size);
 
         TAILQ_INSERT_TAIL(&skm->bufs, sb, sb_link);
@@ -270,7 +269,7 @@ static sockem_buf_t *sockem_buf_add (sockem_t *skm,
  * @remark lock must be held but will be released momentarily while
  *         performing send syscall.
  */
-static int sockem_fwd_bufs (sockem_t *skm, int ofd) {
+static int sockem_fwd_bufs(sockem_t *skm, int ofd) {
         sockem_buf_t *sb;
         int64_t now = sockem_clock();
         size_t to_write;
@@ -278,7 +277,7 @@ static int sockem_fwd_bufs (sockem_t *skm, int ofd) {
 
 
         if (skm->use.direct)
-                to_write = 1024*1024*100;
+                to_write = 1024 * 1024 * 100;
         else if ((elapsed = now - skm->ts_last_fwd)) {
                 /* Calculate how many bytes to send to adhere to rate-limit */
                 to_write = (size_t)((double)skm->use.tx_thruput *
@@ -286,19 +285,18 @@ static int sockem_fwd_bufs (sockem_t *skm, int ofd) {
         } else
                 return 0;
 
-        while (to_write > 0 &&
-               (sb = TAILQ_FIRST(&skm->bufs)) &&
+        while (to_write > 0 && (sb = TAILQ_FIRST(&skm->bufs)) &&
                (skm->use.direct || sb->sb_at <= now)) {
                 ssize_t r;
                 size_t remain = sb->sb_size - sb->sb_of;
-                size_t wr = to_write < remain ? to_write : remain;
+                size_t wr     = to_write < remain ? to_write : remain;
 
                 if (wr == 0)
                         break;
 
                 mtx_unlock(&skm->lock);
 
-                r = send(ofd, sb->sb_data+sb->sb_of, wr, 0);
+                r = send(ofd, sb->sb_data + sb->sb_of, wr, 0);
 
                 mtx_lock(&skm->lock);
 
@@ -312,7 +310,7 @@ static int sockem_fwd_bufs (sockem_t *skm, int ofd) {
                 skm->ts_last_fwd = now;
 
                 sb->sb_of += r;
-                to_write  -= r;
+                to_write -= r;
 
                 if (sb->sb_of < sb->sb_size)
                         break;
@@ -335,7 +333,7 @@ static int sockem_fwd_bufs (sockem_t *skm, int ofd) {
  *
  * @returns the number of bytes forwarded, or -1 on error.
  */
-static int sockem_recv_fwd (sockem_t *skm, int ifd, int ofd, int direct) {
+static int sockem_recv_fwd(sockem_t *skm, int ifd, int ofd, int direct) {
         ssize_t r, wr;
 
         r = recv(ifd, skm->recv_buf, skm->recv_bufsz, MSG_DONTWAIT);
@@ -369,7 +367,7 @@ static int sockem_recv_fwd (sockem_t *skm, int ifd, int ofd, int direct) {
  * @remark Preserves caller's errno.
  * @remark lock must be held.
  */
-static void sockem_close_all (sockem_t *skm) {
+static void sockem_close_all(sockem_t *skm) {
         int serr = socket_errno();
 
         if (skm->ls != -1) {
@@ -392,7 +390,7 @@ static void sockem_close_all (sockem_t *skm) {
  * @brief Copy desired (app) config to internally use(d) configuration.
  * @remark lock must be held
  */
-static __inline void sockem_conf_use (sockem_t *skm) {
+static __inline void sockem_conf_use(sockem_t *skm) {
         skm->use = skm->conf;
         /* Figure out if direct forward is to be used */
         skm->use.direct = !(skm->use.delay || skm->use.jitter ||
@@ -402,9 +400,9 @@ static __inline void sockem_conf_use (sockem_t *skm) {
 /**
  * @brief sockem internal per-socket forwarder thread
  */
-static void *sockem_run (void *arg) {
+static void *sockem_run(void *arg) {
         sockem_t *skm = arg;
-        int cs = -1;
+        int cs        = -1;
         int ls;
         struct pollfd pfd[2];
 
@@ -416,7 +414,7 @@ static void *sockem_run (void *arg) {
         mtx_unlock(&skm->lock);
 
         skm->recv_bufsz = skm->use.recv_bufsz;
-        skm->recv_buf = malloc(skm->recv_bufsz);
+        skm->recv_buf   = malloc(skm->recv_bufsz);
 
         /* Accept connection from sockfd in sockem_connect() */
         cs = accept(ls, NULL, 0);
@@ -426,14 +424,15 @@ static void *sockem_run (void *arg) {
                         /* App socket was closed. */
                         goto done;
                 }
-                fprintf(stderr, "%% sockem: accept(%d) failed: %s\n",
-                        ls, strerror(socket_errno()));
+                fprintf(stderr, "%% sockem: accept(%d) failed: %s\n", ls,
+                        strerror(socket_errno()));
+                mtx_unlock(&skm->lock);
                 assert(cs != -1);
         }
 
         /* Set up poll (blocking IO) */
         memset(pfd, 0, sizeof(pfd));
-        pfd[1].fd = cs;
+        pfd[1].fd     = cs;
         pfd[1].events = POLLIN;
 
         mtx_lock(&skm->lock);
@@ -465,21 +464,19 @@ static void *sockem_run (void *arg) {
                 }
                 mtx_unlock(&skm->lock);
 
-                for (i = 0 ; r > 0 && i < 2 ; i++) {
-                        if (pfd[i].revents & (POLLHUP|POLLERR)) {
+                for (i = 0; r > 0 && i < 2; i++) {
+                        if (pfd[i].revents & (POLLHUP | POLLERR)) {
                                 skm->run = SOCKEM_TERM;
 
                         } else if (pfd[i].revents & POLLIN) {
                                 if (sockem_recv_fwd(
-                                            skm,
-                                            pfd[i].fd,
-                                            pfd[i^1].fd,
-                                            /* direct mode for app socket
-                                             * without delay, and always for
-                                             * peer socket (receive channel) */
-                                            i == 0 ||
-                                            (skm->use.direct &&
-                                             skm->bufs_size == 0)) == -1) {
+                                        skm, pfd[i].fd, pfd[i ^ 1].fd,
+                                        /* direct mode for app socket
+                                         * without delay, and always for
+                                         * peer socket (receive channel) */
+                                        i == 0 || (skm->use.direct &&
+                                                   skm->bufs_size == 0)) ==
+                                    -1) {
                                         skm->run = SOCKEM_TERM;
                                         break;
                                 }
@@ -488,7 +485,7 @@ static void *sockem_run (void *arg) {
 
                 mtx_lock(&skm->lock);
         }
- done:
+done:
         if (cs != -1)
                 sockem_close0(cs);
         sockem_close_all(skm);
@@ -505,8 +502,8 @@ static void *sockem_run (void *arg) {
 /**
  * @brief Connect socket \p s to \p addr
  */
-static int sockem_do_connect (int s, const struct sockaddr *addr,
-                              socklen_t addrlen) {
+static int
+sockem_do_connect(int s, const struct sockaddr *addr, socklen_t addrlen) {
         int r;
 
         r = sockem_connect0(s, addr, addrlen);
@@ -516,7 +513,7 @@ static int sockem_do_connect (int s, const struct sockaddr *addr,
 #ifdef _WIN32
                     && serr != WSAEWOULDBLOCK
 #endif
-                        ) {
+                ) {
 #ifndef _WIN32
                         errno = serr;
 #endif
@@ -528,12 +525,14 @@ static int sockem_do_connect (int s, const struct sockaddr *addr,
 }
 
 
-sockem_t *sockem_connect (int sockfd, const struct sockaddr *addr,
-                          socklen_t addrlen, ...) {
+sockem_t *sockem_connect(int sockfd,
+                         const struct sockaddr *addr,
+                         socklen_t addrlen,
+                         ...) {
         sockem_t *skm;
         int ls, ps;
-        struct sockaddr_in6 sin6 = { .sin6_family = addr->sa_family };
-        socklen_t addrlen2 = addrlen;
+        struct sockaddr_in6 sin6 = {.sin6_family = addr->sa_family};
+        socklen_t addrlen2       = addrlen;
         va_list ap;
 
         pthread_once(&sockem_once, sockem_init);
@@ -574,10 +573,10 @@ sockem_t *sockem_connect (int sockfd, const struct sockaddr *addr,
         }
 
         /* Create sockem handle */
-        skm = calloc(1, sizeof(*skm));
-        skm->as = sockfd;
-        skm->ls = ls;
-        skm->ps = ps;
+        skm                = calloc(1, sizeof(*skm));
+        skm->as            = sockfd;
+        skm->ls            = ls;
+        skm->ps            = ps;
         skm->bufs_size_max = 16 * 1024 * 1024; /* 16kb of queue buffer */
         TAILQ_INIT(&skm->bufs);
         mtx_init(&skm->lock);
@@ -585,10 +584,10 @@ sockem_t *sockem_connect (int sockfd, const struct sockaddr *addr,
         /* Default config */
         skm->conf.rx_thruput = 1 << 30;
         skm->conf.tx_thruput = 1 << 30;
-        skm->conf.delay = 0;
-        skm->conf.jitter = 0;
-        skm->conf.recv_bufsz = 1024*1024;
-        skm->conf.direct = 1;
+        skm->conf.delay      = 0;
+        skm->conf.jitter     = 0;
+        skm->conf.recv_bufsz = 1024 * 1024;
+        skm->conf.direct     = 1;
 
         /* Apply passed configuration */
         va_start(ap, addrlen);
@@ -611,8 +610,8 @@ sockem_t *sockem_connect (int sockfd, const struct sockaddr *addr,
         mtx_unlock(&skm->lock);
 
         /* Connect application socket to listen socket */
-        if (sockem_do_connect(sockfd,
-                              (struct sockaddr *)&sin6, addrlen2) == -1) {
+        if (sockem_do_connect(sockfd, (struct sockaddr *)&sin6, addrlen2) ==
+            -1) {
                 sockem_close(skm);
                 return NULL;
         }
@@ -631,7 +630,7 @@ sockem_t *sockem_connect (int sockfd, const struct sockaddr *addr,
 /**
  * @brief Purge/drop all queued buffers
  */
-static void sockem_bufs_purge (sockem_t *skm) {
+static void sockem_bufs_purge(sockem_t *skm) {
         sockem_buf_t *sb;
 
         while ((sb = TAILQ_FIRST(&skm->bufs)))
@@ -639,7 +638,7 @@ static void sockem_bufs_purge (sockem_t *skm) {
 }
 
 
-void sockem_close (sockem_t *skm) {
+void sockem_close(sockem_t *skm) {
         mtx_lock(&sockem_lock);
         mtx_lock(&skm->lock);
         if (skm->linked)
@@ -648,8 +647,7 @@ void sockem_close (sockem_t *skm) {
 
         /* If thread is running let it close the sockets
          * to avoid race condition. */
-        if (skm->run == SOCKEM_START ||
-            skm->run == SOCKEM_RUN)
+        if (skm->run == SOCKEM_START || skm->run == SOCKEM_RUN)
                 skm->run = SOCKEM_TERM;
         else
                 sockem_close_all(skm);
@@ -672,12 +670,10 @@ void sockem_close (sockem_t *skm) {
  * @remark lock must be held.
  * @returns 0 on success or -1 if key is unknown
  */
-static int sockem_set0 (sockem_t *skm, const char *key, int val) {
-        if (!strcmp(key, "rx.thruput") ||
-            !strcmp(key, "rx.throughput"))
+static int sockem_set0(sockem_t *skm, const char *key, int val) {
+        if (!strcmp(key, "rx.thruput") || !strcmp(key, "rx.throughput"))
                 skm->conf.rx_thruput = val;
-        else if (!strcmp(key, "tx.thruput") ||
-                 !strcmp(key, "tx.throughput"))
+        else if (!strcmp(key, "tx.thruput") || !strcmp(key, "tx.throughput"))
                 skm->conf.tx_thruput = val;
         else if (!strcmp(key, "delay"))
                 skm->conf.delay = val;
@@ -717,7 +713,7 @@ static int sockem_set0 (sockem_t *skm, const char *key, int val) {
 /**
  * @brief Set sockem config parameters
  */
-static int sockem_vset (sockem_t *skm, va_list ap) {
+static int sockem_vset(sockem_t *skm, va_list ap) {
         const char *key;
         int val;
 
@@ -734,7 +730,7 @@ static int sockem_vset (sockem_t *skm, va_list ap) {
         return 0;
 }
 
-int sockem_set (sockem_t *skm, ...) {
+int sockem_set(sockem_t *skm, ...) {
         va_list ap;
         int r;
 
@@ -746,15 +742,15 @@ int sockem_set (sockem_t *skm, ...) {
 }
 
 
-sockem_t *sockem_find (int sockfd) {
+sockem_t *sockem_find(int sockfd) {
         sockem_t *skm;
 
         pthread_once(&sockem_once, sockem_init);
 
         mtx_lock(&sockem_lock);
         LIST_FOREACH(skm, &sockems, link)
-                if (skm->as == sockfd)
-                        break;
+        if (skm->as == sockfd)
+                break;
         mtx_unlock(&sockem_lock);
 
         return skm;
@@ -772,7 +768,7 @@ sockem_t *sockem_find (int sockfd) {
 /**
  * @brief connect(2) overload
  */
-int connect (int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
+int connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
         sockem_t *skm;
 
         pthread_once(&sockem_once, sockem_init);
@@ -787,7 +783,7 @@ int connect (int sockfd, const struct sockaddr *addr, socklen_t addrlen) {
 /**
  * @brief close(2) overload
  */
-int close (int fd) {
+int close(int fd) {
         sockem_t *skm;
 
         pthread_once(&sockem_once, sockem_init);
